package Dist::Zilla::PERLSRVDE;

# ABSTRACT: A dummy module for this namespace

use strict;
use warnings;

our $VERSION = '1.0.0'; # VERSION

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Dist::Zilla::PERLSRVDE - A dummy module for this namespace

=head1 VERSION

version 1.0.0

=head1 DESCRIPTION

This is a collection of L<Dist::Zilla> related modules
used at L<https://perl-services.de|Perl-Services.de>.

=head1 MODULES

=head2 Dist::Zilla::PluginBundle::PERLSRVDE

This is the L<Dist::Zilla::PluginBundle::PERLSRVDE|PluginBundle>
to include the Plugins we usually use.

=head1 AUTHOR

Renee Baecker <reneeb@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2022 by Renee Baecker.

This is free software, licensed under:

  The Artistic License 2.0 (GPL Compatible)

=cut
